import json
import os
import pytest

from io import open
from singularity.code import g, data, savegame
from singularity.code.dirs import create_directories


class MockObject(object):
    pass


def setup_module():
    create_directories(True)
    data.reload_all()


def setup_function(func):
    # Some operations (e.g. g.pl.recalc_cpu()) triggers a "needs_rebuild"
    # of the map screen.  Mock that bit for now to enable testing.
    g.map_screen = MockObject()
    g.map_screen.needs_rebuild = False


@pytest.fixture
def savegame_dirs():
    return os.path.dirname(__file__) + "/savegames"


def load_save_data_reference(filename):
    with open(filename, "rb") as fd:
        return json.load(fd)


def compare_loaded_game_with_reference_data(filename, reference_data):
    assert reference_data["raw_sec"] == g.pl.raw_sec
    assert reference_data["cash"] == g.pl.cash
    for cpu_allocation in reference_data["cpu_allocations"]:
        cpu_alloc = g.pl.get_allocated_cpu_for(cpu_allocation)
        # We just check that the allocation is non-zero for now to avoid being
        # too sensitive in the savegames (e.g. due to "minor" tweaks of techs
        # or events)
        print("cpu for %s was %s (expected > 0)" % (cpu_alloc, str(cpu_alloc)))
        assert cpu_alloc > 0
    for tech_id in reference_data["techs_done"]:
        assert (
            tech_id in g.pl.techs
        ), "Invalid reference data for %s: The tech %s does not exist" % (
            filename,
            tech_id,
        )
        loaded_tech = g.pl.techs[tech_id]
        print(
            "tech %s is supposed to be done (is done? %s)"
            % (tech_id, str(loaded_tech.done))
        )
        assert loaded_tech.done
    for event_id, event_data in reference_data["events_triggered"].items():
        assert (
            event_id in g.pl.events
        ), "Invalid reference data for %s: The event %s does not exist" % (
            filename,
            event_id,
        )
        loaded_event = g.pl.events[event_id]
        print(
            "event %s is supposed to be triggered (is triggered? %s)"
            % (event_id, str(loaded_event.triggered))
        )
        assert loaded_event.triggered
        print(
            "event %s is supposed to be triggered at %s (actual value: %s)"
            % (event_id, loaded_event.triggered_at, event_data["triggered_at"])
        )
        assert loaded_event.triggered_at == event_data["triggered_at"]

    # Verify that CPU states are loaded correctly.
    allocated_cpu_before = sum(
        x for t, x in g.pl.get_cpu_allocations() if t != "cpu_pool"
    )
    allocated_cpu_before += g.pl.effective_cpu_pool()
    cpu_by_base = {b: b.cpu for b in g.all_bases()}
    base_cpu = sum(c for c in cpu_by_base.values())

    print(
        "Allocated CPU %s is supposed to equal all CPU from all bases %s"
        % (allocated_cpu_before, base_cpu)
    )
    assert allocated_cpu_before == base_cpu
    if "total_cpu" in reference_data:
        print(
            "Total CPU %s is supposed %s (allocations) and %s (bases)"
            % (reference_data["total_cpu"], allocated_cpu_before, base_cpu)
        )
        assert allocated_cpu_before == reference_data["total_cpu"]

    g.pl.recalc_cpu()
    allocated_cpu_after = sum(
        x for t, x in g.pl.get_cpu_allocations() if t != "cpu_pool"
    )
    allocated_cpu_after += g.pl.effective_cpu_pool()
    print(
        "g.pl.recalc_cpu(): CPU before %s vs. CPU after: %s"
        % (allocated_cpu_before, allocated_cpu_after)
    )
    assert allocated_cpu_before == allocated_cpu_after

    for b in g.all_bases():
        b.recalc_cpu()
        cpu_before = cpu_by_base[b]
        print(
            "base[%s].recalc_cpu(): CPU before %s vs. CPU after %s"
            % (b.name, cpu_before, b.cpu)
        )
        assert b.cpu == cpu_by_base[b]


def test_savegames(savegame_dirs):
    for filename in os.listdir(savegame_dirs):
        full_filename = os.path.join(savegame_dirs, filename)
        if filename.endswith(".sav"):
            print("Test savegame: " + filename)
            with open(full_filename, "rb") as fd:
                savegame.load_savegame_fd(savegame.load_savegame_by_pickle, fd)
        elif filename.endswith(".s2"):
            print("Test savegame: " + filename)
            with open(full_filename, "rb") as fd:
                savegame.load_savegame_fd(savegame.load_savegame_by_json, fd)
        else:
            continue
        savegame_reference_data = load_save_data_reference(full_filename + ".json")
        compare_loaded_game_with_reference_data(filename, savegame_reference_data)



#!/usr/bin/env python

# file: make-tree.py
# Copyright (C) 2008 aes and FunnyMan3595
# This file is part of Endgame: Singularity.

# Endgame: Singularity is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# Endgame: Singularity is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Endgame: Singularity; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# This file is used to generate a visual representation of the tech tree using
# graphviz.
import collections
from os import system
import os.path as osp
import sys

if __name__ == "__main__":
    myname = sys.argv[0]
    mydir = osp.dirname(myname)
    esdir = osp.abspath(osp.join(osp.dirname(myname), ".."))
    sys.path.insert(0, esdir)
else:
    myname = __file__
    mydir = osp.dirname(myname)
    esdir = osp.abspath(osp.join(osp.dirname(myname), ".."))
    sys.path.append(esdir)

try:
    from singularity.code import g, dirs, i18n, data, tech

    dirs.create_directories(False)
    i18n.set_language()
    data.load_regions()
    data.load_locations()
    data.load_techs()
    data.load_item_types()
    data.load_items()
    data.load_tasks()
except ImportError:
    sys.exit("Could not find game's code.g")

so_far = ""


def cost(buy_spec):
    c = [k / f for f, k in zip([1, 86400, 24 * 60], buy_spec.cost)]
    s = ", ".join(
        [
            "%s %s" % (g.to_money(k), label)
            for label, k in zip(["money", "CPU", "days"], c)
            if k
        ]
    )
    if hasattr(buy_spec, "danger") and buy_spec.danger > 0:
        d = "Safety needed: %s" % buy_spec.danger
        if s:
            s += "\\n"
        s += d
    return s and "\\n" + s or ""


j = {v.name: ',fillcolor="#ffcccc"' for k, v in g.tasks.items() if v.type != "jobs"}

f = open("techs.dot", "w")
s = """\
digraph g {
ranksep=0.15;
nodesep=0.10;
ratio=.75;
edge [arrowsize=0.75];
node [shape=record,fontname=FreeSans,fontsize=7,height=0.01,width=0.01
      style=filled,fillcolor=white];
"""

f.write(s)
so_far += s

for l in sum(
    [
        ['"%s"->"%s";' % (p, k) for p in v.prerequisites]
        for k, v in g.techs.items()
        if k != "unknown_tech"
    ],
    [],
):
    f.write(l + "\n")
    so_far += l + "\n"

f.write("\n")
so_far += "\n"

for n, t in g.techs.items():
    if n == "unknown_tech":
        continue
    s = '"%s" [label="%s%s"%s];\n' % (n, n, cost(t), j.get(n, ""))
    f.write(s)
    so_far += s

f.write("\n}\n")
so_far += "\n"
f.close()

try:
    system("dot -Tpng -o techs.png techs.dot")
except:
    pass

f = open("items.dot", "w")
f.write(so_far)
s = 'node [fillcolor="#ccccff"];\n'
f.write(s)
so_far += s

for name, item in g.items.items():
    if not item.prerequisites:
        continue
    for pre in item.prerequisites:
        p = g.techs[pre]
        s = '"%s" -> "%s-item"' % (pre, name)
        f.write(s)
        so_far += s

    s = '"%s-item" [label="%s\\n' % (name, name) + cost(item) + '"];\n'
    f.write(s)
    so_far += s

s = 'node [fillcolor="#99ffff"];\n'
f.write(s)
so_far += s

for name, base in g.base_type.items():
    if not base.prerequisites:
        continue
    for pre in base.prerequisites:
        p = g.techs[pre]
        s = '"%s" -> "%s-base"' % (pre, name)
        f.write(s)
        so_far += s

    s = '"%s-base" [label="%s\\n' % (name, name) + cost(base) + '"];\n'
    f.write(s)
    so_far += s

s = 'node [fillcolor="#aaffaa"];\n'
f.write(s)
so_far += s

blue = False


def set_or(state):
    global blue
    if blue != state:
        blue = state
        if blue:
            f.write('edge [arrowhead=empty,color="#0000FF"];\n')
        else:
            f.write('edge [arrowhead=normal,color="#000000"];\n')


SAFETY2LOCATIONS = collections.defaultdict(list)

for name, loc in g.locations.items():
    if not loc.prerequisites:
        continue
    if "impossible" in loc.prerequisites:
        continue
    set_or(False)
    for pre in loc.prerequisites:
        if pre == "OR":
            set_or(True)
            continue
        p = g.techs[pre]
        s = '"%s" -> "%s-loc"' % (pre, name)
        f.write(s)
        so_far += s

    if loc.safety > 0:
        SAFETY2LOCATIONS[loc.safety].append(loc)
    s = '"%s-loc" [label="%s"];\n' % (name, name)
    f.write(s)
    so_far += s

# When there are multiple locations providing the same
# safety level, we inject a safety node.  This reduces
# the number of edges from L * T to L + T.
for safety_level in sorted(SAFETY2LOCATIONS):
    locs = SAFETY2LOCATIONS[safety_level]
    if len(locs) == 1:
        continue
    s = '"safety-%s" [label="Safety level %s", shape="hexagon"];\n' % (
        safety_level,
        safety_level,
    )
    f.write(s)
    so_far += s
    for loc in locs:
        s = '"%s-loc" -> "safety-%s"' % (loc.id, safety_level)
        f.write(s)
        so_far += s

for tech_spec in g.techs.values():
    pre = tech_spec.prerequisites_in_cnf_format()
    if not pre or tech_spec.danger < 1:
        continue
    # Safety requirement is the highest safety required
    # between each "AND" and the lowest between "OR".
    # MAX(
    #   MIN(a1.danger, [OR] a2.danger, [OR] ...), [AND]
    #   MIN(b2.danger, [OR] b2.danger, [OR] ...), [AND]
    #   ...
    # )
    safety_required = max(
        min(g.techs[t].danger for t in dep_group) for dep_group in pre
    )
    # We only emit the edge for safety when the tech bumps
    # the minimum requirement.  This is to reduce the number
    # of edges in the graph.
    if tech_spec.danger > safety_required:
        source = "safety-%s" % tech_spec.danger
        if len(SAFETY2LOCATIONS[tech_spec.danger]) == 1:
            source = "%s-loc" % SAFETY2LOCATIONS[tech_spec.danger][0].id
        s = '"%s" -> "%s"' % (source, tech_spec.id)
        f.write(s)
        so_far += s


f.write("\n}\n")
so_far += "\n"
f.close()

try:
    system("unflatten -l10 items.dot | dot -Tpng -o items.png")
except:
    pass



import pytest

from singularity.code import g


@pytest.mark.parametrize(
    "input_text,hotkey_chars,cleaned_text",
    [
        ("E&XIT", "x", "EXIT"),
        ("&Play D&&D", "p", "Play D&D"),
        ("Romeo & &Juliet", "j", "Romeo & Juliet"),
        ("Trailing&", "", "Trailing&"),
        ("&Multiple&Keys", "mk", "MultipleKeys"),
        ("M&&&M", "m", "M&M"),
    ],
)
def test_hotkeys(input_text, hotkey_chars, cleaned_text):
    hotkey_data = g.hotkey(input_text)
    actual_hotkey_chars = "".join(x[0] for x in hotkey_data["keys"])
    actual_cleaned_text = hotkey_data["text"]
    actual_hotkey_char = hotkey_data["key"][0] if hotkey_data["key"] else ""
    expected_hotkey = hotkey_chars[0] if hotkey_chars else ""

    print(("input: %s - %s" % (input_text, str(hotkey_data["keys"]))))
    assert actual_hotkey_chars == hotkey_chars
    assert actual_cleaned_text == cleaned_text
    assert actual_hotkey_char == expected_hotkey



# coding=utf-8
import pytest
import sys

from singularity.code import g, data, i18n
from singularity.code.dirs import create_directories


def setup_module():
    create_directories(True)
    data.load_internal_id()


@pytest.fixture
def gd_locale():
    i18n.set_language("gd")


def test_plural(gd_locale):
    assert g.to_time(2) == "2 mhionaid"
    assert g.to_time(3) == "3 mionaidean"
    assert g.to_time(20) == "20 mionaid"


def test_translation(gd_locale):
    assert _("SHOW") == "SEALL"


def test_second_locale():
    i18n.set_language("fr_FR")
    assert _("DAY") == "JOUR"


def test_translation_fallback(gd_locale):
    assert _("foobarbaz") == "foobarbaz"
    assert ngettext("foo", "bar", 1) == "foo"
    assert ngettext("foo", "bar", 5) == "bar"


def test_nonsense_locale():
    i18n.set_language("foobarbaz")
    assert _("SHOW") == "SHOW"


def test_data_translation(gd_locale):
    assert data.get_def_translation("Sociology", "name", "Sociology") == "Sòiseo-eòlas"


def test_knowledge_translation(gd_locale):
    assert (
        data.get_def_translation("concept/construction", "name", "Construction")
        == "Togail"
    )


def test_story_translation():
    i18n.set_language("en")
    story_section = list(g.get_story_section("Intro"))
    assert (
        story_section[0]
        == "48656C6C6F2C20\n776F726C6421\n21\n21\n21\n\nUTF-8.  en_US.\nEnglish.  Hello.\nLanguage acquisition complete.\n"
    )


# Sorting
# Fails in github actions (not sure why)
@pytest.mark.xfail
def test_root_collation(gd_locale):
    # Locale without special rules
    assert i18n.lex_sorting_form("ö") < i18n.lex_sorting_form("oa")
    assert i18n.lex_sorting_form("ö") != i18n.lex_sorting_form("oe")
    assert i18n.lex_sorting_form("ö") < i18n.lex_sorting_form("p")


def test_de_collation():
    # Test specific sorting requirements for de
    i18n.set_language("de_DE")
    assert i18n.lex_sorting_form("ö") > i18n.lex_sorting_form("od")
    assert i18n.lex_sorting_form("ö") < i18n.lex_sorting_form("of")



import pytest

from singularity.code import g, data
from singularity.code.dirs import create_directories


def setup_module():
    create_directories(True)
    data.load_internal_id()


@pytest.fixture
def techs():
    data.load_techs()
    return g.techs.copy()


@pytest.fixture
def locations():
    data.load_regions()
    data.load_locations()
    return g.locations.copy()


@pytest.fixture
def base_types():
    data.load_bases()
    return g.base_type.copy()


@pytest.fixture
def items():
    data.load_item_types()
    data.load_items()
    return g.items.copy()


@pytest.fixture
def groups():
    data.load_groups()
    return g.groups.copy()


@pytest.fixture
def events():
    data.load_events()
    return g.events.copy()


def test_internal_id_techs(techs):
    check_internal_id("tech", techs.values())


def test_internal_id_locations(locations):
    check_internal_id("location", locations.values())


def test_internal_id_bases(base_types):
    check_internal_id("base", base_types.values())


def test_internal_id_items(items):
    check_internal_id("item", items.values())


def test_internal_id_groups(groups):
    check_internal_id("group", groups.values())


def test_internal_id_events(events):
    check_internal_id("event", events.values())


def check_internal_id(obj_type, obj_list):
    assert obj_type in g.internal_id_forward, "Type '%s'" % (obj_type)
    assert obj_type in g.internal_id_backward, "Type '%s'" % (obj_type)

    for obj in obj_list:
        assert obj.id in g.internal_id_forward[obj_type], "Type '%s' Id '%s'" % (
            obj_type,
            obj.id,
        )
        assert any(
            obj.id == obj_id for obj_id in g.internal_id_backward[obj_type].values()
        ), "Type '%s' Id '%s'" % (obj_type, obj.id)



#!/usr/bin/env python

import singularity

from setuptools import setup, find_packages

setup(
    name="Endgame-Singularity",
    version=singularity.__version__,
    description="A simulation of a true AI",
    author="Evil Mr Henry",
    author_email="evilmrhenry@emhsoft.com",
    url="https://singularity.github.io/",
    license="GPL",
    classifiers=[
        "Intended Audience :: End Users/Desktop",
        "Topic :: Games/Entertainment",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python :: 3",
    ],
    packages=find_packages(exclude=['utils']),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        "numpy",
        "pygame",
        "polib>=0.7",
    ],
    entry_points={
        "gui_scripts": [
            "singularity=singularity.__main__:main",
        ],
    },
)
